#!/bin/bash
source $(dirname $0)/../../config
: ${NS2:=$NS2DIR/ns}
: ${SEED:=9999}
: ${DURATION:=60}
: ${FLOWTRACE:=$OUTDIR/flows.tr}
# : ${LINKTRACE:=$OUTDIR/link.tr}
# : ${COMPTRACE:=$OUTDIR/comp.tr}
: ${BSPEED:=50}
: ${BDELAY:=10}
: ${MAXDELAY:=100}
: ${QUIET:=0}

[ $# -lt 2 ] && {
	echo "usage: $0 <queue> <traffic> [<traffic>...]"
	echo "  queue := ABE|DSF|FIFO|SPRI|VIFQ"
	echo "  traffic := tcp|tcpx|tfrc|cbr|poisson|pareto|sflow|web <args>"
	echo "  args := [<load>] <target> <conns> <start> <psize>"
	exit 1
}

case "$1" in
ABE|DSF|FIFO|SPRI|VIFQ)
	bqueue=$1;;
*)
	echo "unknown queue: " $1; exit 1;;
esac
shift

cmd="$NS2 $(dirname $0)/dsf.tcl -seed $SEED -quiet $QUIET
-bspeed $BSPEED -bdelay $BDELAY -bqueue $bqueue -maxdelay $MAXDELAY
-stopTime $DURATION -flowtracename $FLOWTRACE"

[ ! -z "$LINKTRACE" ] && cmd="$cmd -linktracename $LINKTRACE"
[ ! -z "$COMPTRACE" ] && cmd="$cmd -comptracename $COMPTRACE"

i=0
while [ $# -gt 0 ]; do
	traffic[$i]=$1
	case "$1" in
	tcp)
		cmd="$cmd -traffic$i TCP";;
	tcpx)
		cmd="$cmd -traffic$i TCPX";;
	tfrc)
		cmd="$cmd -traffic$i TFRC";;
	cbr)
		cmd="$cmd -traffic$i CBR -load$i $2"; shift;;
	poisson)
		cmd="$cmd -traffic$i Poisson -load$i $2"; shift;;
	pareto)
		cmd="$cmd -traffic$i Pareto -load$i $2"; shift;;
	paretoShape)
		if [ "$2" = "p" ]; then
			cmd="$cmd -traffic$i Poisson -load$i $3"
		else
			cmd="$cmd -traffic$i Pareto -load$i $3 -shape$i $2"
		fi; shift 2;;
	sflow)
		cmd="$cmd -traffic$i ShortSimple";;
	web)
		cmd="$cmd -traffic$i ShortWeb";;
	*)
		echo "unknown traffic type: " $1; exit 1;;
	esac
	shift
	if echo $1|fgrep -q . || [ $1 -ge 0 ]; then
		cmd="$cmd -dtarget$i $1"
	else
		cmd="$cmd -sclass$i $(expr 0 - $1 - 1)"
	fi
	cmd="$cmd -connections$i $2 -start$i $3 -psize$i $4"
	shift 4
	i=$(expr $i + 1)
done
cmd="$cmd -tgroups $i"

eval "$(echo $cmd)"
