#ifndef _DSF_QUEUE_H
#define _DSF_QUEUE_H 1

#include "ns3/packet.h"
#include "ns3/ptr.h"
#include "ns3/queue.h" 
#include "ns3/simulator.h"

#include <iostream>
#include <fstream>
#include <list>
#include <map>
#include <string> 

using namespace std;

namespace ns3 {

// HACK ALARM!
class DSFQueue;
typedef int64_t PacketTime;
#define DSFPACKET Ptr<Packet>
#define DSFQ DSFQueue*
#include "ns3/dsfbase.h"

template<typename T>
static inline void terminate( const T& m1 ) {
  cerr << "FATAL: " << m1 << endl;
  exit(1);
}

template<typename T, typename U>
static inline void terminate( const T& m1, const U& m2  ) {
  cerr << "FATAL: " << m1 << ' ' << m2 << endl;
  exit(1);
}

struct DSFClass {
  unsigned int num;
  double delay;
};

class DSFQueue : public Queue {
public:
  static const unsigned int monMax = 64;
  unsigned int monCount;
private:
  struct Monitor {
    struct Data {
      uint64_t barrival;
      uint64_t bservice;
      uint64_t parrival;
      uint64_t pdropped;
      uint64_t ldropped;
      uint64_t pservice;
      double delaySum;
      double delaySum2;
      double delayMax;
      void clear() {
        barrival = bservice = parrival = pdropped = ldropped = pservice = 0;
        delaySum = delaySum2 = delayMax = 0;
      }
      Data() { clear(); }
    } cur, tot;
    uint64_t ivals;
    void arrival( uint64_t b ) { cur.barrival += b; cur.parrival += 1; }
    void drop   ( uint64_t b, bool late ) {
      cur.pdropped += 1;
      if (late) cur.ldropped += 1;
    }
    void service( uint64_t b, double d ) {
      cur.bservice += b;
      cur.pservice += 1;
      cur.delaySum += d;
      cur.delaySum2 += (d*d);
      if (d > cur.delayMax) cur.delayMax = d;
    }
    void interval() {
      tot.barrival += cur.barrival;
      tot.bservice += cur.bservice;
      tot.parrival += cur.parrival;
      tot.pdropped += cur.pdropped;
      tot.ldropped += cur.ldropped;
      tot.pservice += cur.pservice;
      tot.delaySum += cur.delaySum;
      tot.delaySum2 += cur.delaySum2;
      if (cur.delayMax > tot.delayMax) tot.delayMax = cur.delayMax;
      ivals += 1;
      cur.clear();
    }
    void summary() {
      tot.barrival /= ivals;
      tot.bservice /= ivals;
    }
    Monitor() : ivals(0) {}
  } mon[monMax];

  uint64_t droptrain;
  uint64_t svctrain;

  inline bool classify(Ptr<Packet> p, uint32_t& sc, uint32_t& tc);

  DSFBase base;
  double linkRate;
  ofstream traceFile;
  ofstream monFile;
  ofstream monFilePkt;
  ofstream monFileDrop;
  double monInterval;
  bool initialized;
public:
  static TypeId GetTypeId (void);
  DSFQueue();
  virtual ~DSFQueue();
  void setLinkRate(double lr) { linkRate = lr; }
  void setQueueLimit(int ql) { base.setQueueLimit(ql); }
  void AddClass( const DSFClass& c ) { base.addClass( c.num, (linkRate * c.delay) / 8 ); }
  void setLD();
  void setABE();
  void setPRIO();
  void setTraceFile( const string& filename );
  void startMonitor( const string& filename, const string& filenamePkt, const string& filenameDrop, double start, double interval, unsigned int nodeCount );
  void stopMonitor();
  inline void tprint(char* buf, Monitor::Data& m);
  inline void monitor();

  static void drop( DSFQueue* This, Ptr<Packet> p, bool late = false );
  static void trace( DSFQueue* This, char* buf );
  static void reset( DSFQueue* This ) {
    This->droptrain = This->svctrain = 0;
  }

private:
  virtual bool DoEnqueue (Ptr<Packet> p);
  virtual Ptr<Packet> DoDequeue (void);
  virtual Ptr<const Packet> DoPeek (void) const;
};

}; // namespace

#endif /* _DSF_QUEUE_H */
