library(data.table)
library(doParallel)

## process command-line arguments
args <- commandArgs(TRUE)
inputfile = "codeloutput.csv"
inputfolder = "~/tmp/tmixanalysis/codel/"
inputfolder = args[1]
outputfolder <- "~/tmp/nn/nnsimulations2/data"
outputfolder <- args[2]

outputfileCodel <- 'codel'
outputfileDroptail <- 'droptail'



## check parts of configuration ("assert")
stopifnot(length(args)==2)



###################
tlast <- function(x) {
    nn <- length(x)
    return(x[nn])
}

clean <- function(x) as.numeric(as.character(x))

ci <- function(x) {
  t=try(t.test(x,conf.level=0.975), silent=TRUE)
  if (is(t, "try-error")) return(0) else return(round(t$estimate-t$conf.int[1],4))
}
################

inputpath <- paste(inputfolder,inputfile,sep='/')
csvfile <- data.table(read.csv(inputpath,header=F))

processFile2 <- function(csvfile) {
    setkey(csvfile,V1,V2,V3)
    cdelays <- csvfile[grep("-",V4),list(name=V1,d=as.numeric(unlist(strsplit(as.character(V4),"-"))[2*(1:length(V4))]),v=clean(V5))]
    cdelays=cdelays[,list(v=sum(v)),by=list(name,d)]
    cdelays[,p:=v/sum(v),by=name]
    cdelays[,ecdf:=cumsum(p),by=name]
    # change unit from s to ms
    cdelays[,d:=d*1000]
    return(data.table(q75=cdelays[round(ecdf,2)<.75,last(d),by=name]$V1,
                      q99=cdelays[round(ecdf,2)<.99,last(d),by=name]$V1,
                      max=cdelays[v>1,max(d),by=name]$V1,
                      x=cdelays[,name,by=name]))
}

delays = processFile2(csvfile)



processFile5 <- function(csvfile) {
    setkey(csvfile,V1,V2)
    tp=csvfile[V4=="CMP",list(name=V1,run=V2,tp=clean(V3)/clean(V5))]
    return(tp[,list(mtp=mean(tp)),by=list(x.name=name)])
}

goodput = processFile5(csvfile)


############

setkey(delays,x.name)
setkey(goodput,x.name)

# droptail
tt <- delays[grep("Droptail",x.name)][goodput[grep("Droptail",x.name)]]
setkey(tt,mtp)

# runs for different delays
tt[,delay:=unlist(strsplit(as.character(x.name),"-"))[2*(1:length(x.name))]]
tt[,x.name:=sub("-10ms|-30ms|-80ms","",x.name)]

ttt <- tt[,list(q99=min(q99)),by=list(mtp=round(mtp/5),delay)]
tt[,mtporig:=mtp]
tt[,mtp:=round(mtp/5)]
setkey(tt,mtp,delay,q99)
setkey(ttt,mtp,delay,q99)

tttt <- tt[ttt][,list(mtporig=tlast(round(mtporig/125)),q99=tlast(q99),max=tlast(max),name=tlast(x.name)),by=list(mtp,delay)]
setkey(tttt,mtporig,delay)

err <- foreach(dd = c("10ms","30ms","80ms")) %do% {
# fitting a 2-polynomial
    tmp <- tttt[delay==dd]
    fit99 <- loess(q99 ~ mtporig, degree=2, data=tmp[q99<quantile(q99,.74)])
    j99 <- order(tmp[,mtporig])
    fitmax <- loess(max ~ mtporig, data=tmp[max<quantile(max,.76)])
    jmax <- order(tmp[,mtporig])
    texp <- tmp[,list(mtporig,q99,fit99=fit99$fitted[j99],max,max99=fitmax$fitted[jmax])]
    write.table(texp, paste(outputfolder,"/",outputfileDroptail,"-",dd,".table",sep=""), quote=F, col.names=F, row.names=F)
}




# codel
tt <- delays[grep("CoDel",x.name)][goodput[grep("CoDel",x.name)]]
setkey(tt,mtp)

# runs for different delays
tt[,delay:=unlist(strsplit(as.character(x.name),"-"))[5*(1:length(x.name))]]
tt[,x.name:=sub("-10ms|-30ms|-80ms","",x.name)]

ttt <- tt[,list(q99=min(q99)),by=list(mtp=round(mtp/10000),delay)]
tt[,mtporig:=mtp]
tt[,mtp:=round(mtp/10000)]
setkey(tt,mtp,delay,q99)
setkey(ttt,mtp,delay,q99)

tttt <- tt[ttt][,list(mtporig=tlast(round(mtporig/125)),q99=tlast(q99),max=tlast(max),name=tlast(x.name)),by=list(mtp,delay)]
setkey(tttt,mtporig,delay)

err <- foreach(dd = c("10ms","30ms","80ms")) %do% {
# fitting a 2-polynomial
    tmp <- tttt[delay==dd]
    fit99 <- loess(q99 ~ mtporig, data=tmp)
    j99 <- order(tmp[,mtporig])
    fitmax <- loess(max ~ mtporig, data=tmp)
    jmax <- order(tmp[,mtporig])
    texp <- tmp[,list(mtporig,q99,fit99=fit99$fitted[j99],max,max99=fitmax$fitted[jmax])]
    write.table(texp, paste(outputfolder,"/",outputfileCodel,"-",dd,".table",sep=""), quote=F, col.names=F, row.names=F)
}







# default codel
tt <- delays[grep("CoDel",x.name)][goodput[grep("CoDel",x.name)]]
setkey(tt,mtp)

# runs configs..
tt[,delay:=unlist(strsplit(as.character(x.name),"-"))[5*(1:length(x.name))]]
tt[,qlen:=unlist(strsplit(as.character(x.name),"-"))[5*(1:length(x.name))-3]]
tt[,interval:=unlist(strsplit(as.character(x.name),"-"))[5*(1:length(x.name))-2]]
tt[,target:=unlist(strsplit(as.character(x.name),"-"))[5*(1:length(x.name))-1]]
tt[,x.name:=sub("-10ms|-30ms|-80ms","",x.name)]

dtt <- tt[qlen=="750" & target=="5"]

dttt <- dtt[,list(q99=min(q99)),by=list(mtp=round(mtp/1),delay)]
dtt[,mtporig:=mtp]
dtt[,mtp:=round(mtp/1)]
setkey(dtt,mtp,delay,q99)
setkey(dttt,mtp,delay,q99)

dtttt <- dtt[dttt][,list(mtporig=tlast(round(mtporig/125)),q99=tlast(q99),max=tlast(max),name=tlast(x.name)),by=list(mtp,delay)]
setkey(dtttt,mtporig,delay)




err <- foreach(dd = c("10ms","30ms","80ms")) %do% {
# fitting a 2-polynomial
    tmp <- dtttt[delay==dd]
    fit99 <- loess(q99 ~ mtporig, data=tmp)
    j99 <- order(tmp[,mtporig])
    fitmax <- loess(max ~ mtporig, data=tmp)
    jmax <- order(tmp[,mtporig])
    texp <- tmp[,list(mtporig,q99,fit99=fit99$fitted[j99],max,max99=fitmax$fitted[jmax])]
    write.table(texp, paste(outputfolder,"/default",outputfileCodel,"-",dd,".table",sep=""), quote=F, col.names=F, row.names=F)
}




